#
# Copyright (c) 2008-2018 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'

from persistent import Persistent
from zope.container.contained import Contained
from zope.interface import implementer, provider
from zope.schema.fieldproperty import FieldProperty

from onf_website.component.hearing import IHearingTarget
from onf_website.component.location import ILocationTarget
from onf_website.shared.resource.interfaces import IResource, IResourceInfo, IWfResource, IWfResourceFactory, \
    RESOURCE_CONTENT_NAME, RESOURCE_CONTENT_TYPE, RESOURCE_INFO_ANNOTATIONS_KEY
from pyams_content.component.illustration.interfaces import IIllustrationTarget, ILinkIllustrationTarget
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget
from pyams_content.component.theme.interfaces import ICollectionsTarget, ITagsTarget, IThemesTarget
from pyams_content.features.preview.interfaces import IPreviewTarget
from pyams_content.features.review.interfaces import IReviewTarget
from pyams_content.shared.common import SharedContent, register_content_type
from pyams_content.shared.common.interfaces import IWfSharedContentFactory
from pyams_content.shared.common.types import WfTypedSharedContent
from pyams_utils.adapter import adapter_config, get_annotation_adapter


@implementer(IWfResource, IIllustrationTarget, ILinkIllustrationTarget, IParagraphContainerTarget,
             ILocationTarget, IHearingTarget, ITagsTarget, ICollectionsTarget, IThemesTarget,
             IPreviewTarget, IReviewTarget)
class WfResource(WfTypedSharedContent):
    """Resource class"""

    content_type = RESOURCE_CONTENT_TYPE
    content_name = RESOURCE_CONTENT_NAME


register_content_type(WfResource)


@implementer(IResourceInfo)
class ResourceInfo(Persistent, Contained):
    """Resource persistent informations"""

    editor = FieldProperty(IResourceInfo['editor'])
    author = FieldProperty(IResourceInfo['author'])
    release_year = FieldProperty(IResourceInfo['release_year'])
    nb_pages = FieldProperty(IResourceInfo['nb_pages'])
    editor_reference = FieldProperty(IResourceInfo['editor_reference'])
    isbn_number = FieldProperty(IResourceInfo['isbn_number'])
    price = FieldProperty(IResourceInfo['price'])
    summary = FieldProperty(IResourceInfo['summary'])
    publisher_words = FieldProperty(IResourceInfo['publisher_words'])


@adapter_config(context=IWfResource, provides=IResourceInfo)
def resource_info_factory(context):
    """Resource info factory"""
    return get_annotation_adapter(context, RESOURCE_INFO_ANNOTATIONS_KEY, ResourceInfo, name='++info++')


@provider(IWfResourceFactory)
@implementer(IResource)
class Resource(SharedContent):
    """Workflow managed resource"""


@adapter_config(context=IWfResourceFactory, provides=IWfSharedContentFactory)
def resource_content_factory(context):
    return WfResource
